/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "agps.h"
#include "sv_bar.h"

// TODO:
// 1)  Connect the stats items to real GPS activities

void notebook_expose(const char* pageName);
static void page_stats_fix_time_update(int x);
enum
{
    MASK_TTF = 1,
    MASK_TTFF = 2
};

static GtkLabel* wid_ttff;
static GtkLabel* wid_ttf;

class FixTime
{
public:
    void init();
    void fix_time(float t);
    char* report(int n, char* buf) const;

    FixTime(const char* name) : name_(name)
    {
        init();
    };

private:
    int n_;          // Number of samples

    double min_;
    double max_;

    double mean_;
    double S_;

    const char* name_;
};

void
FixTime::init()
{
    min_  = 99999.0;
    max_  = 0.0;
    n_    = 0;
    mean_ = 0.0;
    S_    = 0.0;
}

FixTime ttf ("TTF" );           // Time to fix
FixTime ttff("TTFF");           // Time to first fix

void
FixTime::fix_time(float t)
{
    ++n_;

    // printf("%s += %.1f %d\n", name_, t, n_);

    double delta = t - mean_;
    mean_ += delta / (double) n_;
    S_    += delta * (t - mean_);

    if (t < min_) min_ = t;
    if (t > max_) max_ = t;
}

char*
FixTime::report(int n, char* buf) const
{
    if (n_ < 1)
    {
        sprintf(buf,    "%s(%d)   Min: - s    Max: - s\n"
                        "Average: -        std -    ",
            name_, n_);
    }
    else
    {
        sprintf(buf,    "%s(%d)   Min: %.1f s    Max: %.1f s\n"
                        "Average: %.1f s     std %.1f",
            name_, n_,
            min_, max_, mean_, sqrt(S_/n_));
    }
    return buf;
}

void
page_stats_fix_time(float TTF, float TTFF)
{
    if (TTFF > 0.0)
    {
        ttff.fix_time(TTFF);
        page_stats_fix_time_update(MASK_TTFF);
    }
    else if (TTF > 0.0)
    {
        ttf.fix_time(TTF);
        page_stats_fix_time_update(MASK_TTF);
    }
}

void
page_stats_init()
{
    ttf.init();
    ttff.init();
    page_stats_fix_time_update(MASK_TTFF | MASK_TTF);
}

static void
page_stats_fix_time_report(FILE* f)
{
    char buf_ttf [128];
    char buf_ttff[128];

    fprintf(f, "Position Fix Time\n%s\n%s\n",
                ttf.report (sizeof(buf_ttf ), buf_ttf ),
                ttff.report(sizeof(buf_ttff), buf_ttff) );
}

void
page_stats_report(FILE* f)
{
    // report the average position, CEP, etc.
    page_stats_fix_time_report(f);
}

static void
page_stats_fix_time_update(int x)
{
    char buf[128];

    if (MASK_TTF & x)
    {
        ttf.report(sizeof(buf), buf);
        gtk_label_set_text(wid_ttf, buf);
    }

    if (MASK_TTFF & x)
    {
        ttff.report(sizeof(buf), buf);
        gtk_label_set_text(wid_ttff, buf);
    }
}

void page_stats_done()
{
    // page_stats_position_update();
    // page_stats_position_least_squares_update();
    page_stats_fix_time_update(MASK_TTFF | MASK_TTF);
}

static gboolean
expose_stats_event(GtkWidget* w, GdkEventExpose* event)
{
    notebook_expose("ST");
    return FALSE;
}


GtkWidget*
page_stats_create(GtkWidget* window)
{
    GtkWidget* label;
    GtkWidget* table;

    //--------------------------------------------------------------------------
    //          table
    //--------------------------------------------------------------------------
    table = gtk_table_new(14, 2, TRUE);
    gtk_signal_connect(GTK_OBJECT(table), "expose_event",
                    G_CALLBACK(expose_stats_event), NULL);

    //--------------------------------------------------------------------------
    //          Position Average
    //--------------------------------------------------------------------------
    label = gtk_label_new("Position Average");
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 2, 0, 1);
    gtk_widget_show(label);

        //----------------------------------------------------------------------
        //          Position Average : Lat..StdDev
        //----------------------------------------------------------------------
        label = gtk_label_new("Lat:   37.123456 N");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 1, 2);
        gtk_widget_show(label);
        label = gtk_label_new("StdDev: 0.0 m ");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 1, 2, 1, 2);
        gtk_widget_show(label);

        //----------------------------------------------------------------------
        //          Position Average : Lon..StdDev
        //----------------------------------------------------------------------
        label = gtk_label_new("Lon:  121.654321 W");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 2, 3);
        gtk_widget_show(label);
        label = gtk_label_new("StdDev: 0.0 m ");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 1, 2, 2, 3);
        gtk_widget_show(label);

        //----------------------------------------------------------------------
        //          Position Average : Alt..StdDev
        //----------------------------------------------------------------------
        label = gtk_label_new("Alt:  123 m");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 3, 4);
        gtk_widget_show(label);
        label = gtk_label_new("StdDev: 0.0 m ");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 1, 2, 3, 4);
        gtk_widget_show(label);

    //--------------------------------------------------------------------------
    //          Least Squares Average
    //--------------------------------------------------------------------------
    label = gtk_label_new("Least Squares Average");
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 2, 4, 5);
    gtk_widget_show(label);

        //----------------------------------------------------------------------
        //          Least Squares Average : Lat..StdDev
        //----------------------------------------------------------------------
        label = gtk_label_new("Lat:   37.123456 N");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 5, 6);
        gtk_widget_show(label);
        label = gtk_label_new("StdDev: 0.0 m ");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 1, 2, 5, 6);
        gtk_widget_show(label);

        //----------------------------------------------------------------------
        //          Least Squares Average : Lon..StdDev
        //----------------------------------------------------------------------
        label = gtk_label_new("Lon:  121.654321 W");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 6, 7);
        gtk_widget_show(label);
        label = gtk_label_new("StdDev: 0.0 m ");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 1, 2, 6, 7);
        gtk_widget_show(label);

        //----------------------------------------------------------------------
        //          Least Squares Average : Alt..StdDev
        //----------------------------------------------------------------------
        label = gtk_label_new("Alt:  123 m");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 1, 7, 8);
        gtk_widget_show(label);
        label = gtk_label_new("StdDev: 0.0 m ");
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
        gtk_table_attach_defaults(GTK_TABLE (table), label, 1, 2, 7, 8);
        gtk_widget_show(label);

    //--------------------------------------------------------------------------
    //          Position Fix Time
    //--------------------------------------------------------------------------
    label = gtk_label_new("Position Fix Time");
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
    gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 2, 9, 10);
    gtk_widget_show(label);

    //----------------------------------------------------------------------
    //          Position Fix Time : TTF
    //----------------------------------------------------------------------
    label = gtk_label_new("");
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 2, 10, 12);
    gtk_widget_show(label);
    wid_ttf = GTK_LABEL(label);

    //----------------------------------------------------------------------
    //          Position Fix Time : TTFF
    //----------------------------------------------------------------------
    label = gtk_label_new("");
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 2, 12, 14);
    gtk_widget_show(label);
    wid_ttff = GTK_LABEL(label);

    page_stats_init();

    gtk_widget_show(table);
    return table;
}
